package ACL2::Proceso::Generalizacion;
use base ACL2::Proceso;

=head1 NAME

ACL2::Proceso::Generalizacion - ACL2 generaliza una conjetura para simplificar su enunciado y facilitar el razonamiento

=head1 FUNCTIONS

=cut

#
#    Clase Generalizacion: proceso de demostración de ACL2
#    Copyright (C) 2006-2008 Antonio García Domínguez
#
#    This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

use strict;
use warnings;
use ACL2::Orden;
use ACL2::Proceso::Fabrica;
use ACL2::Localizacion;

our $lh = ACL2::Localizacion->get_handle();

BEGIN {
    ACL2::Proceso::Fabrica->registrar
	(
	 __PACKAGE__,
	 sub {
	     return ACL2::Proceso::Generalizacion->_identificar(@_);
	 }
	 );
}
sub escribir_xml_cabecera {
    my ($this, $writer) = @_;

    $writer->startTag(ACL2::Tags::GENERALIZE_TAG);

    # Reemplazos
    $writer->startTag(ACL2::Tags::REPLACEMENTS_TAG);
    while (my ($origen,$destino) = each(%{$this->{rh_reempl}})) {
	$writer->emptyTag(ACL2::Tags::REPLACEMENT_TAG, 
			  "from" => $origen, 
			  "to" => $destino);
    }
    $writer->endTag(ACL2::Tags::REPLACEMENTS_TAG);
    
    if (exists $this->{restrict_variable}) {
	$writer->emptyTag(ACL2::Tags::RESTRICTVAR_TAG,
			  "variable" => $this->{restrict_variable});
    }

    ACL2::Proceso->volcar_reglas($writer, $this->{reglas}) 
	if exists $this->{reglas};

    ACL2::Orden->salida_parrafos($writer, $this->{salida});
}

sub escribir_xml_final {
    my ($this, $writer) = @_;
    $writer->endTag(ACL2::Tags::GENERALIZE_TAG);
}

sub _identificar
{
    my ($paquete, $salida) = @_;
    my %res      = ();

    $salida =~ /^\s*We generalize this conjecture, ([^\.]+?)\./o 
	or return undef;

    (undef, my %terms) = split /\s*(?:and )?(replacing|restricting|established)\s*/o, $1 
	or die $lh->maketext("No se pudieron obtener las acciones de la generalización");

    # Reemplazos
    $res{rh_reempl} = ACL2::Proceso->obtener_reemplazos($terms{replacing});

    # Restricciones
    if (exists $terms{restricting}) {
	$terms{restricting} =~ /the new variable (.*) to be that/ 
	    or die $lh->maketext("No se pudo obtener la variable.");
	$res{restrict_variable} = $1;
    }

    # Reglas usadas
    $res{reglas} = ACL2::Proceso->obtener_reglas($terms{established}) 
	if (exists $terms{established});

    # Salida
    $res{salida} = substr $salida, $-[0], $+[0] - $-[0];

    return \%res;
}

1;
